@@ -6,7 +6,7 @@ class urls :
         self.base_paper_url = 'https://act.webullbroker.com/webull-paper-center/api'
         self.base_quote_url = 'https://quoteapi.webullbroker.com/api'
         self.base_securities_url = 'https://securitiesapi.webullbroker.com/api'
-        self.base_trade_url = 'https://tradeapi.webulltrade.com/api/trade'
+        self.base_trade_url = 'https://tradeapi.webullbroker.com/api/trade'
         self.base_user_url = 'https://userapi.webull.com/api'
         self.base_userbroker_url = 'https://userapi.webullbroker.com/api'
 
@@ -110,7 +110,7 @@ class urls :
         return f'{self.base_trade_url}/v2/corder/stock/place/{account_id}'
 
     def quotes(self, stock):
-        return f'{self.base_quote_url}/quote/tickerRealTimes/v5/{stock}'
+        return f'{self.base_options_gw_url}/quotes/ticker/getTickerRealTime?tickerId={stock}&includeSecu=1&includeQuote=1'
 
     def rankings(self):
         return f'{self.base_securities_url}/securities/market/v5/6/portal'
@@ -140,4 +140,4 @@ class urls :
         return f'{self.base_user_url}/social/feed/topic/{topic}/posts?size={num}'
 
     def social_home(self, topic, num=100):
-        return f'{self.base_user_url}/social/feed/topic/{topic}/home?size={num}'
\ No newline at end of file
+        return f'{self.base_user_url}/social/feed/topic/{topic}/home?size={num}'
